package messif.objects.annotation;

import java.util.Objects;
import net.didion.jwnl.data.Synset;

/**
 * Component of the semantic graph. This object represents a semantic link between
 * synsets in the graph. Each edge is of a defined type and contains weight that 
 * is passed through the network.
 * @author Honza Botorek
 */
public class SynsetEdge {
    private final double edgeValue;
    private final SynsetRelationType edgeType;
    private final Synset synsetTarget;

    public SynsetEdge(double edgeValue, Synset synsetTarget, SynsetRelationType edgeType) {
        this.edgeValue = edgeValue;
        this.synsetTarget = synsetTarget;
        this.edgeType = edgeType;
    }

    public double getEdgeValue() {
        return edgeValue;
    }

    public Synset getEndVertex() {
        return synsetTarget;
    }

    public long getEndVertexLocator() {
        return synsetTarget.getOffset();
    }

    public SynsetRelationType getEdgeType() {
        return edgeType;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 29 * hash + Objects.hashCode(this.edgeType);
        hash = 29 * hash + Objects.hashCode(this.synsetTarget);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SynsetEdge other = (SynsetEdge) obj;
        if (!Objects.equals(this.edgeType, other.edgeType)) {
            return false;
        }
        if (!Objects.equals(this.synsetTarget, other.synsetTarget)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return  edgeType + "  :   "+ edgeValue + "; target = " + synsetTarget.getOffset() + " ("+synsetTarget+")" ;
    }
}
