package messif.objects.annotation;

import java.util.Objects;

/**
 * Convenient object that maps IndexedWord object into its offset. 
 * It is utilized in the WordNet frequancy computation process.
 * @author Honza Botorek
 */
public class WordNetFrequencyObject {
    private String indexedSenseWord;
    private long indexedSenseOffset;

    public WordNetFrequencyObject(String indexedSenseWord, long indexedSenseOffset) {
        this.indexedSenseWord = indexedSenseWord;
        this.indexedSenseOffset = indexedSenseOffset;
    }

    public String getIndexedSenseWord() {
        return indexedSenseWord;
    }

    public void setIndexedSenseWord(String indexedSenseWord) {
        this.indexedSenseWord = indexedSenseWord;
    }

    public long getIndexedSenseOffset() {
        return indexedSenseOffset;
    }

    public void setIndexedSenseOffset(long indexedSenseOffset) {
        this.indexedSenseOffset = indexedSenseOffset;
    }

    @Override
    public int hashCode() {
        int hash = 87;
        hash = 87* hash + Objects.hashCode(this.indexedSenseWord);
        hash = 87 * hash + (int) (this.indexedSenseOffset ^ (this.indexedSenseOffset >>> 32));
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        WordNetFrequencyObject other = (WordNetFrequencyObject) obj;
        return this.indexedSenseWord.equals(other.indexedSenseWord) &&
                this.indexedSenseOffset == other.indexedSenseOffset;
    }
}
