package messif.objects.annotation.api;

import messif.objects.annotation.Annotation;
import messif.objects.annotation.AnnotationTerm;
import messif.objects.classification.ClassificationException;
import messif.objects.classification.Classifier;
import messif.utility.Parametric;

/**
 * Creates an {@link Annotation} from a given object.
 * 
 * @param <T> the type of object that this annotator accepts as input
 * @param <O> the type of annotation that this annotator provides as output
 * @author Michal Batko <batko@fi.muni.cz>
 */
public interface Annotator<T, O extends Annotation> extends Classifier<T, AnnotationTerm> {
    @Override
    public O classify(T object, Parametric parameters) throws ClassificationException;
    /**
     * Returns the class of {@link Annotation} that this processor accepts as input.
     * @return the class of {@link Annotation} that this processor accepts as input
     */
    public Class<? extends T> getAnnotationInputClass();
    /**
     * Returns the class of {@link Annotation} that this processor provides as output.
     * @return the class of {@link Annotation} that this processor provides as output
     */
    public Class<? extends O> getAnnotationOutputClass();
}
