package messif.objects.annotation.impl;

import messif.objects.annotation.Annotation;
import messif.objects.annotation.AnnotationTerm;
import messif.objects.annotation.api.AnnotationTransformer;
import messif.objects.classification.ClassificationException;
import messif.utility.Parametric;

/**
 * Transformer component that computes (inefficiently) frequency-based probabilities
 * of input Annotation terms.
 * @author Honza Botorek
 */
public class AnnotationFrequencyProbabilityTransformer implements AnnotationTransformer<Annotation, Annotation>{

    @Override
    public Annotation classify(Annotation inputAnnotation, Parametric parameters) throws ClassificationException {
        Annotation resultAnnotation = new Annotation(inputAnnotation.getAnnotatedObject());
        double extWordsSum = 0;
        for(AnnotationTerm at : inputAnnotation){
            extWordsSum += at.getOriginFrequency();
        }
        for(AnnotationTerm at : inputAnnotation){
            double initProbab = at.getOriginFrequency();
            double normProD = initProbab / extWordsSum;
            resultAnnotation.add(at);
            resultAnnotation.updateConfidence(at, (float)normProD);
        }
        return resultAnnotation;
    }

    @Override
    public Class<? extends Annotation> getAnnotationInputClass() {
        return Annotation.class;
    }

    @Override
    public Class<? extends Annotation> getAnnotationOutputClass() {
        return Annotation.class;
    }

    @Override
    public Class<? extends AnnotationTerm> getCategoriesClass() {
        return AnnotationTerm.class;
    }
    
}
