package messif.objects.annotation.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import messif.objects.annotation.AnnotationTerm;
import messif.objects.annotation.AnnotationWithSynsets;
import messif.objects.annotation.AnnotationWithSynsetsEdges;
import messif.objects.annotation.SynsetVertex;
import messif.objects.annotation.api.AnnotationReducer;
import messif.objects.classification.ClassificationException;
import messif.utility.Parametric;

/**
 * This selector method selects only terms that were presented in the initial
 * visual search - e.g. words that were parts of annotations of visually similar
 * images of the input image.
 * According to the most probable vertices from the semantic network, corresponding
 * Annotation terms are selected.
 *
 * @author Honza Botorek
 */
public class CandidateSelectionReducer implements AnnotationReducer<AnnotationWithSynsetsEdges, AnnotationWithSynsets> {
    private final int bestResultsLimit;

    public CandidateSelectionReducer(int bestResultsLimit) {
        this.bestResultsLimit = bestResultsLimit;
    }

    @Override
    public AnnotationWithSynsets classify(AnnotationWithSynsetsEdges sourceClassification, Parametric prmtrc) throws ClassificationException {
        List<SynsetVertex> vertices = SynsetVertex.sortByProbability(sourceClassification.getSemanticNetwork().keySet());

        // Copy the best terms for the first "best-results" vertices
        AnnotationWithSynsets aws = new AnnotationWithSynsets(sourceClassification.getAnnotatedObject());
        int count = 0;
        for (SynsetVertex synVert : vertices) {
            AnnotationTerm term = sourceClassification.getBestTermForSynset(synVert.getSynset());
            if (term != null) {
                aws.updateConfidence(term, (float) synVert.getVertexProbability());
                aws.addTermSynset(term, synVert.getSynset());
                count++;
                if (count >= bestResultsLimit)
                    break;
            }
        }
        return aws;
    }

    @Override
    public Class<? extends AnnotationTerm> getCategoriesClass() {
        return AnnotationTerm.class;
    }

    @Override
    public Class<? extends AnnotationWithSynsetsEdges> getAnnotationInputClass() {
        return AnnotationWithSynsetsEdges.class;
    }

    @Override
    public Class<? extends AnnotationWithSynsets> getAnnotationOutputClass() {
        return AnnotationWithSynsets.class;
    }
}
