package messif.objects.annotation.impl;

import java.util.HashMap;
import java.util.Map;
import messif.objects.annotation.Annotation;
import messif.objects.annotation.AnnotationTerm;
import messif.objects.annotation.AnnotationWithSynsetsEdges;
import messif.objects.annotation.SynsetVertex;
import messif.objects.annotation.api.AnnotationReducer;
import messif.objects.classification.ClassificationException;
import messif.utility.Parametric;
import net.didion.jwnl.data.Synset;
import net.didion.jwnl.data.Word;

/**
 * Reducer component that selects categories from VCO ontology from the 
 * final semantic network. 
 * @author Honza Botorek
 */
public class ClassSelectorReducer implements AnnotationReducer<AnnotationWithSynsetsEdges, Annotation> {

    @Override
    public Annotation classify(AnnotationWithSynsetsEdges sourceClassification, Parametric parameters) throws ClassificationException {   
        Annotation ann = new Annotation(sourceClassification.getAnnotatedObject());  
        Map<AnnotationTerm, Synset> annTermToSynsetMap = new HashMap<>();
        for(SynsetVertex vertex : sourceClassification.getSemanticNetwork().keySet()){
            if(vertex.getSynset().getOffset() >= 999990000){
                Word[] words = vertex.getSynset().getWords();
                AnnotationTerm at = new AnnotationTerm(constructNameFromCategory(words));
                ann.add(at);
                ann.updateConfidence(at, (float)vertex.getVertexProbability());
                annTermToSynsetMap.put(at, vertex.getSynset());
            }
        }        
        return ann;
    }

    @Override
    public Class<? extends AnnotationWithSynsetsEdges> getAnnotationInputClass() {
        return AnnotationWithSynsetsEdges.class;
    }

    @Override
    public Class<? extends Annotation> getAnnotationOutputClass() {
        return Annotation.class;
    }

    @Override
    public Class<? extends AnnotationTerm> getCategoriesClass() {
        return AnnotationTerm.class;
    }

    /**
     * Method constructs a string representation of the given synset.
     * According to the Word objects (originated from WordNet) the string
     * is constructed characterizing the input synset.
     * @param words array of synset original strings
     * @return string representation of input Word objects
     */
    private String constructNameFromCategory(Word[] words) {
        StringBuilder sb = new StringBuilder();
        for(int i = 0; i < words.length; i++){
            String lemma = words[i].getLemma();
            sb.append(lemma.replace("VIRT_", ""));
            if(i != words.length-1)
                sb.append(", ");
        }
        return sb.toString();
    }
}
