package messif.objects.annotation.impl;

import java.util.Iterator;
import java.util.TreeMap;
import messif.objects.annotation.AnnotationTerm;
import messif.objects.annotation.AnnotationWithSynsetsEdges;
import messif.objects.annotation.SynsetVertex;
import messif.objects.annotation.api.AnnotationReducer;
import messif.objects.classification.ClassificationException;
import messif.utility.Parametric;

/**
 * Reducer component that limits initial amount of vertices that form the Semantic 
 * Network. 
 * 
 * @author Honza Botorek
 */
public class LimitInitialVerticesReducer implements AnnotationReducer<AnnotationWithSynsetsEdges, AnnotationWithSynsetsEdges> {
    private final int CONST_INITIAL_VERTICES_NUMBER;

    public LimitInitialVerticesReducer(int CONST_INITIAL_VERTICES_NUMBER) {
        this.CONST_INITIAL_VERTICES_NUMBER = CONST_INITIAL_VERTICES_NUMBER;
    }
    
    @Override
    public AnnotationWithSynsetsEdges classify(AnnotationWithSynsetsEdges sourceAnnotation, Parametric prmtrc) throws ClassificationException {
        AnnotationWithSynsetsEdges resultAnnotation = new AnnotationWithSynsetsEdges(sourceAnnotation, sourceAnnotation.getTermSynsets());
        
        TreeMap<Double, Integer> sortedProbabilitiesWithCounts = new TreeMap<Double, Integer>();
        Iterator<SynsetVertex> it = sourceAnnotation.getSemanticNetwork().keySet().iterator();
        while (it.hasNext()) {
            Double vertexProbability = it.next().getVertexProbability();
            if (!sortedProbabilitiesWithCounts.containsKey(vertexProbability)) {
                sortedProbabilitiesWithCounts.put(vertexProbability, 0);
            }
            sortedProbabilitiesWithCounts.put(vertexProbability,sortedProbabilitiesWithCounts.get(vertexProbability)+1);
        }
        //get the threshold value
        Iterator<Double> keyIterator = sortedProbabilitiesWithCounts.descendingKeySet().iterator();
        int i = 0;
        double thresholdValue = 0;
        double sumOfProbabilitiesAboveThreshold = 0;
        while (keyIterator.hasNext() && i< CONST_INITIAL_VERTICES_NUMBER) {            
            double value = keyIterator.next();
            i+=sortedProbabilitiesWithCounts.get(value);
            thresholdValue = value;
            sumOfProbabilitiesAboveThreshold += value*sortedProbabilitiesWithCounts.get(value);
        }     
        while (keyIterator.hasNext()) {
            double value = keyIterator.next();
            if (value == thresholdValue) {
                sumOfProbabilitiesAboveThreshold += value*sortedProbabilitiesWithCounts.get(value);
            }
        }
        it = sourceAnnotation.getSemanticNetwork().keySet().iterator();
        while (it.hasNext()) {
            SynsetVertex v = it.next();
            if (v.getVertexProbability() >= thresholdValue) {
                v.setVertexProbability(v.getVertexProbability()/sumOfProbabilitiesAboveThreshold);
                resultAnnotation.addVertexToNetwork(v);
            }
        }
        return resultAnnotation;
    }

    @Override
    public Class<? extends AnnotationTerm> getCategoriesClass() {
        return AnnotationTerm.class;
    }

    @Override
    public Class<? extends AnnotationWithSynsetsEdges> getAnnotationInputClass() {
        return AnnotationWithSynsetsEdges.class;
    }

    @Override
    public Class<? extends AnnotationWithSynsetsEdges> getAnnotationOutputClass() {
        return AnnotationWithSynsetsEdges.class;
    }
    
}
