package messif.objects.annotation.impl;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import messif.objects.annotation.Annotation;
import messif.objects.annotation.AnnotationTerm;
import messif.objects.annotation.api.AnnotationExpander;
import messif.objects.classification.ClassificationException;
import messif.utility.Parametric;

/**
 * Extender component utilizing statistic information of language provided
 * by UK UFAL.
 * Class is able to provide a certain amount of similar words to the query one
 * based on their corpus co-occurence.
 * 
 * @author Honza Botorek
 */
public class StatisticSimilarWordsExtender implements AnnotationExpander<Annotation, Annotation>{
    private final int constPecinaStatsSimilarNumber;
    private final double constPecinaCoOccImportanceRatio;
    private Connection conn;
    private String connUrl;

    public StatisticSimilarWordsExtender(int constPecinaStatsSimilarNumber, double constPecinaCoOccImportanceRatio, String connSt) throws SQLException {
        this.constPecinaStatsSimilarNumber = constPecinaStatsSimilarNumber;
        this.constPecinaCoOccImportanceRatio = constPecinaCoOccImportanceRatio;
        connUrl = connSt;
        this.conn = DriverManager.getConnection(connSt);;
    }

    @Override
    public Annotation classify(Annotation inputAnnotation, Parametric parameters) throws ClassificationException {
        Annotation resultAnnotation = new Annotation(inputAnnotation);
        try {
            if(conn == null) conn = DriverManager.getConnection(connUrl);
            for(AnnotationTerm at : inputAnnotation){            
                PreparedStatement s = conn.prepareStatement("SELECT neighbor,probability FROM `profimedia_keywordphrase_neighbors`"
                        + " WHERE word = ? limit ?");
                s.setString(1, at.getWord());
                s.setInt(2, constPecinaStatsSimilarNumber);
                s.executeQuery();
                ResultSet rt = s.getResultSet();
                while (rt.next()) {
                    String word = rt.getString(1);
                    double probab = rt.getDouble(2);
                    AnnotationTerm extensionTerm = new AnnotationTerm(word);
                    double computedProbab = at.getOriginFrequency() * probab * constPecinaCoOccImportanceRatio;
                    resultAnnotation.add(extensionTerm, (float)computedProbab);
                }
            }
        
        } catch (SQLException e) {
            throw new ClassificationException("There was an error when extending by similar words: " + e, e);
        }
        closeCoOcDatabaseConnection();
        return resultAnnotation;
    }
    
    
    public void closeCoOcDatabaseConnection(){
        if(conn!=null){
            try {
                conn.close();
            } catch (SQLException ex) {
            }
        }
    }

    @Override
    public Class<? extends Annotation> getAnnotationInputClass() {
        return Annotation.class;
    }

    @Override
    public Class<? extends Annotation> getAnnotationOutputClass() {
        return Annotation.class;
    }

    @Override
    public Class<? extends AnnotationTerm> getCategoriesClass() {
        return AnnotationTerm.class;
    }
    
}
